#   File: comb.R  07/03/2024
#   
#   This program constructs the contingency table in De Gooijer and Zerom 
#   (2000) and then calculates 
#   the two tests of independence: S_n and Chi^2. Moreover, it 
#   calculates the forecast error measures. 
#      
#   Reference:
#   De Gooijer, J.G. and Zerom, D. (2000).
#      Kernel based multi-step-ahead prediction of the U.S. short-term interest
#      rate. Journal of Forecasting, 19(4), 335-353.      
##################################################################
#  Read the forecast output from one of the methods. The file
#  contains five columns as the maximum forecast horizon is 
#  mx = five. The required outputs are generated from the program files
#  "allmean.R, allmedia.R, and allmode.R".         
###################################################################
library(matrixcalc)  # Used to vectorize a matrix

# predicted values (mode, mean, median) 
data1        = prd
zn           = data1;

# Read also the actual values 
data         = act
zn1          = data;

mx = 5
df = 254     
j  = 1     

# k1 and k2 are in terms of the real values 
k1 = zn1- matrix(1,df,mx)*10       
k2 = zn - matrix(1,df,mx)*10        
h1 = k1*k2       

# 'g' means greater than zero and 'l' means less than zero 

nn  = matrix(0,4,4)     
gg1 = 0     
gz1 = 0     
gl1 = 0     
zg1 = 0     
zz1 = 0     
zl1 = 0     
lg1 = 0     
lz1 = 0     
ll1 = 0     

i = 1     
while (i <= df){     
  if ((k1[i,j]>0) & (k2[i,j]>0)){     
     gg = 1     
  } else {     
     gg = 0     
  }     

  if ((k1[i,j]>0) & (k2[i,j]==0)){     
     gz = 1     
  } else {     
     gz=0     
  }     

  if ((k1[i,j]>0) & (k2[i,j]<0)){     
     gl = 1     
  } else {     
     gl = 0     
  }    

  if ((k1[i,j]==0) & (k2[i,j]>0)){     
    zg = 1     
  } else {    
    zg = 0     
  }     

  if ((k1[i,j]==0) & (k2[i,j]==0)){     
    zz = 1     
  } else {    
    zz = 0     
  }   

  if ((k1[i,j]==0) & (k2[i,j]<0)){     
    zl = 1     
  } else {    
    zl = 0     
  }   

  if ((k1[i,j]<0) & (k2[i,j]>0)){     
    lg = 1     
  } else {     
    lg = 0     
  }    

  if ((k1[i,j]<0) & (k2[i,j]==0)){     
    lz = 1     
  } else {    
    lz = 0     
  }   

  if ((k1[i,j]<0) & (k2[i,j]<0)){     
    ll = 1     
  } else {    
    ll = 0     
  }    

  gg1 = gg1+gg     
  gz1 = gz1+gz     
  gl1 = gl1+gl     
  zg1 = zg1+zg     
  zz1 = zz1+zz     
  zl1 = zl1+zl     
  lg1 = lg1+lg     
  lz1 = lz1+lz     
  ll1 = ll1+ll     
  i   = i+1  
  
}  # end while     

# The addition of a '1' is to avoid the occurence of zeros 
# 'g' means greater than zero and 'l' means less than zero 

nn[1,1] = gg1     
nn[1,2] = gz1+1     
nn[1,3] = gl1     
nn[2,1] = zg1+1     
nn[2,2] = zz1+1     
nn[2,3] = zl1+1     
nn[3,1] = lg1     
nn[3,2] = lz1+1     
nn[3,3] = ll1     

nn[1,4] = nn[1,1]+nn[1,2]+nn[1,3]       #  n10 
nn[2,4] = nn[2,1]+nn[2,2]+nn[2,3]       #  n20 
nn[3,4] = nn[3,1]+nn[3,2]+nn[3,3]       #  n30 
nn[4,1] = nn[1,1]+nn[2,1]+nn[3,1]       #  n01 
nn[4,2] = nn[1,2]+nn[2,2]+nn[3,2]       #  n02 
nn[4,3] = nn[1,3]+nn[2,3]+nn[3,3]       #  n03 
nn[4,4] = nn[1,4]+nn[2,4]+nn[3,4]       # last row is col-sum [3 * 3] matrix
                                        # last col is row-sum
n       = nn[4,4]                       # [4 * 4] 

# C_{ij} table (i=1,2,3) rows are actual moves in the series
#              (j=1,2,3) columns predicted moves
#
# diagonal cells = correct sign prediction
# of diagonal cells = incorrect prediction

## nn; Table II in paper for one-step ahead (m = 1)
##     [,1] [,2] [,3] [,4]
## [1,]   86    1   10   97
## [2,]   17    1    3   21
## [3,]   82    1   58  141
## [4,]  185    3   71  259
 
    
# Calculate the proportions that are used to build the test statistics 
# pp = percentage of correct sign prediction = sum of the diagonal elements
#      divided by the sum of all elements

pp = matrix(0,4,4)     
i  = 1     
while (i <= 4){     
  j = 1     
  while (j <= 4){     
     pp[i,j] = nn[i,j]/n     
     j       = j+1     
  }
  i = i+1     
}

# Correct prediction under the null 
pnul = pp[1,4]*pp[4,1]+pp[2,4]*pp[4,2]+pp[3,4]*pp[4,3] # pnul = 0.4176891   
     
# Observed correct prediction: hat{P} see Table
pest = pp[1,1]+pp[2,2]+pp[3,3] # pest = 0.5598456 for conditional mean  
 
SS   = pest - pnul     # pred - observed
pp1  = pp[1:3,1:3]     
pp2  = vec(t(pp1))     
pp3  = pp2 %*% t(pp2)       # pp' [9 * 9]  

pt = c(pp2)  # transform in column vector
dim(diag(9))
pp4 = diag(pt, 9, 9) # replace the diagonal of a matrix by "pt"; the big phi 

part = matrix(0,3,3)     

i = 1     
while (i <= 3){     
  j = 1     
  while (j <= 3){     
    if (i==j){     
       part[i,j] = 1-pp[4,i]-pp[i,4]     
    } else {    
       part[i,j] = -pp[j,4]-pp[4,i]     
    }   
    j = j+1     
  } 
   i = i+1     
}

part = vec(t(part))     
VV   = t(part) %*% (pp4-pp3) %*% part     
sn   = sqrt(n) %*% ( 1/sqrt(VV) ) %*% SS  # Eq (17) in JDG & Z (2000)  
SS1  = sn*sn   # The SS^2_n test statistics follows a Chi^2_{(3-2)}  
               # SS1=35.38234; 5% nominal value = 3.84 df =1

# Chi^2 test "n2" has df(3-1)^2 = 4. Critical value = 9.49 at 5% level     

#  Computation of the chi-square test "n2" of independence 

n2 = 0     
e  = 1     
while (e <= 3){     
   n1 = 0     
   f  = 1     
   while (f <= 3){     
      a  = (nn[e,4] %*% nn[4,f])/n     
      n0 = (nn[e,f]-a)^2/a     
      n1 = n1+n0     
      f  = f+1     
   }
   n2 = n2+n1     
   e  = e+1     
} 

## n2 = 31.74557


# Computation of error measures for m-step ahead

err    = (zn1-zn)                           #  Just errors  
errnp  = abs(zn1-zn)                        #  Absolute error  
errnps = (zn1-zn)^2                         #  Squared error  
errnpr = (abs(zn1-zn)/abs(zn1))*100         #  Relative absolute eror   

# After the computation of these quantities,
#  calculate the mean or median of them   

MSE   = colMeans(errnps)     
MAE   = colMeans(errnp)     
MedSE = median(errnps)     
MedAE = median(errnp)                  

#########################################################




